"""PyPi Configuration File Manager."""

__author__ = "Greg Albrecht <gba@splunk.com>"
__copyright__ = "Copyright 2012 Splunk, Inc."
__license__ = "Apache License 2.0"

import os
from configparser import ConfigParser


class PyPiRC:
    """
    PyPi Configuration File Manager.

    Can be used for updating ~/.pypirc file programatically.

        Example::
        >>> a = PyPiRC('doctest_pypi.cfg')
        >>> new_server = {'pypi': {'repository': 'http://pypi.example.com'}}
        >>> new_server2 = {'pypi2': {'repository': 'http://pypi2.example.com'}}
        >>> a.servers.update(new_server)
        >>> a.servers.update(new_server2)
        >>> a.save()
        >>> 'pypi' in a.servers
        True
        >>> 'pypi2' in a.servers
        True
    """

    RC_FILE = os.path.join(os.path.expanduser("~"), ".pypirc")  # noqa: PTH111,PTH118

    def __init__(self, rc_file: str | None = None) -> None:
        if rc_file is None:
            self.rc_file = self.RC_FILE
        else:
            self.rc_file = rc_file

        self.conf = ConfigParser()
        if os.path.exists(self.rc_file):  # noqa: PTH110
            self.conf.read(self.rc_file)

        self._create_distutils()
        self._servers: dict[str, dict[str, str]] = {}
        for server in self._get_index_servers():
            if self.conf.has_section(server):
                server_conf = {server: dict(self.conf.items(server))}
                self.servers.update(server_conf)

    def _create_distutils(self) -> None:
        """Creates top-level distutils table in pypirc."""
        if not self.conf.has_section("distutils"):
            self.conf.add_section("distutils")

    def save(self) -> None:
        """Saves pypirc file with new configuration information."""
        for server, conf in iter(self.servers.items()):
            self._add_index_server()
            for conf_k, conf_v in iter(conf.items()):
                if not self.conf.has_section(server):
                    self.conf.add_section(server)
                self.conf.set(server, conf_k, conf_v)

        with open(self.rc_file, "w", encoding="utf8") as f:  # noqa: PTH123
            self.conf.write(f)
        self.conf.read(self.rc_file)

    def _get_index_servers(self) -> list[str]:
        """Get index-servers currently configured in pypirc."""
        idx_srvs = []
        if "index-servers" in self.conf.options("distutils"):
            idx = self.conf.get("distutils", "index-servers")
            idx_srvs = [srv.strip() for srv in idx.split("\n") if srv.strip()]
        return idx_srvs  # noqa: RET504

    def _add_index_server(self) -> None:
        """Add index-server to distutil's `index-servers` param."""
        index_servers = "\n\t".join(self.servers.keys())
        self.conf.set("distutils", "index-servers", index_servers)

    @property
    def servers(self) -> dict[str, dict[str, str]]:
        """index-servers configured in pypirc."""
        return self._servers

    @servers.setter
    def servers(self, server: dict[str, dict[str, str]]) -> None:
        """Adds index-servers to pypirc."""
        self._servers.update(server)
        self._servers.update(server)
        self._servers.update(server)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pypirc']

package_data = \
{'': ['*']}

install_requires = \
['prettyprinter>=0.18.0,<0.19.0']

entry_points = \
{'console_scripts': ['pypirc = pypirc.cmd:main']}

setup_kwargs = {
    'name': 'pypirc-voidei',
    'version': '1.1.0',
    'description': 'PyPiRC: .pypirc Manager',
    'long_description': '<!-- markdownlint-disable MD026 MD036-->\n\n# `.pypirc` File Management Client\n\n**"This is hopefully a temporary fork, as the official project looks dead."**\nsaid [Chappers](https://github.com/charliec443), who originally forked this project. Unfortunately, that fork is also dead.\nSo here I am. Forking it again, making it work with Python 3.10+ (no backwards compatibility).\n\nIf you want to have it work in older versions of Python, please go download the [original](https://github.com/ampledata/pypirc), or [Chapper\'s fork](https://github.com/charliec443/pypirc).\n\n## Installation\n\nInstall from [pypi](https://pypi.org) via the following:\n\n```bash\npip install pypirc-voidei\n```\n\n## Usage Example\n\nDisplay current pypi configuration:\n\n```bash\npypirc\n```\n\nAdd a pypi server:\n\n```bash\npypirc -s mypypi -u foo -p bar -r https://mypypi.example.com/\n```\n\n### Credits:\n\n#### Source\n\nOfficial: <https://github.com/ampledata/pypirc>\nChappers: <https://github.com/chappers/pypirc>\nvoidei: <https://github.com/voidei/pypirc>\n\n#### Author\n\nGreg Albrecht <gba@splunk.com>\n<http://ampledata.org/>\n\n#### Copyright\n\n```plaintext\nCopyright 2012 Splunk, Inc.\n```\n\n#### License\n\n**Apache License 2.0**\n',
    'author': 'Greg Albrecht',
    'author_email': 'gba@splunk.com',
    'maintainer': 'Dawn Walker',
    'maintainer_email': 'dawniepieuwu@gmail.com',
    'url': 'https://github.com/voidei/pypirc',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.11,<4',
}


setup(**setup_kwargs)



"""PyPiRC CLI Client."""

__author__ = "Greg Albrecht <gba@splunk.com>"
__copyright__ = "Copyright 2012 Splunk, Inc."
__license__ = "Apache License 2.0"

import argparse

from prettyprinter import cpprint  # pyright: ignore

from pypirc.pypirc import PyPiRC


def main() -> None:
    """Main."""
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "-s",
        "--server",
        help="Index Server Name",
        metavar="SERVER",
    )
    parser.add_argument(
        "-r",
        "--repository",
        help="Repository URL",
        metavar="URL",
    )
    parser.add_argument(
        "-u",
        "--username",
        help="User Name",
        metavar="USERNAME",
    )
    parser.add_argument(
        "-p",
        "--password",
        help="Password",
        metavar="PASSWORD",
    )
    parser.add_argument(
        "-q",
        "--quiet",
        help="Quiet mode",
        default=True,
        action="store_false",
    )

    options = parser.parse_args()

    myrc = PyPiRC()

    if options.server:
        server = myrc.servers.get(options.server, {}) if myrc.servers else {}

        if options.repository:
            server["repository"] = options.repository
        if options.username:
            server["username"] = options.username
        if options.password:
            server["password"] = options.password

        myrc.servers[options.server] = server
        myrc.save()

    if not options.quiet:
        pass
    elif myrc.servers:
        cpprint(myrc.servers)
    else:
        print(".pypirc empty!")
